(PackageInfo named: 'ContextS2') preamble: '"below, add code to be run before the loading of this package"

"Thread local storage for thread-local layer activation.
Is there a way to store inst vars of a class in Monticello extensions??"

(Process instVarNames includes: #activeLayers) 
	ifFalse: [Process addInstVarName: ''activeLayers'' ]

'!
SystemOrganization addCategory: #ContextS2!
SystemOrganization addCategory: #'ContextS2-Tests'!
SystemOrganization addCategory: #'ContextS2-Tests-Sandbox'!
(PackageInfo named: 'ContextS2') postscript: '"ContextS2 overwrites Behavior>>#basicAddSelector:withMethod: (extensions).
Since the method is used by Monticello it has to be added after the load (ContextS2PostLoad)."

| repository project repo |

repository := ''http://www.hpi.uni-potsdam.de/hirschfeld/squeaksource/''.
project := ''ContextSTrunk''.

repo := MCRepositoryGroup default repositories detect: 
	[ :each | (each respondsTo: #locationWithTrailingSlash) and: [each locationWithTrailingSlash = (repository, project, ''/'')]].

((Smalltalk at: #InstallerMonticello ifAbsent: [self error: ''You need InstallerMonticello to load (Version 4.0)'']) new 
	http: repository user: repo user password: repo password)
	project: project;
	install: ''ContextS2PostLoad''.'!

!CompiledMethod methodsFor: '*ContextS2' stamp: 'ag 12/21/2011 11:24'!
asPartialMethod
	
	| layer |
	layer := (self pragmas
		detect: [ :each | each keyword == CSLayeredMethod pragmaKeyword ]
		ifNone: [ nil ])
		ifNotNilDo: [ :pragma |
			pragma arguments size == 1 ifFalse: [ self error: 'Illegal ContextS2-Pragma: Need only one argument.' ].
			layer := pragma argumentAt: 1.
			(layer isSymbol | (layer isArray and: [ layer allSatisfy: #isSymbol ]))
				ifFalse: [ self error: 'Illegal ContextS2-Pragma: Need Symbol or Array of Symbols as parameter.' ].
			layer isArray ifTrue: [ layer ] ifFalse: [ Array with: layer ] ].
	^ CSPartialMethod
				on: layer
				forMethod: self! !

!CompiledMethodWithNode class methodsFor: '*ContextS2' stamp: 'ag 12/20/2011 21:39'!
generateMethodFromNode: aMethodNode trailer: bytes
	
	| method |
	method := aMethodNode generate: bytes.
	(method pragmas anySatisfy: [:each | each keyword = CSLayeredMethod pragmaKeyword ])
		ifTrue: [method := method asPartialMethod].
	^ self method: method node: aMethodNode.! !

!Parser class methodsFor: '*ContextS2' stamp: 'ag 12/20/2011 21:39'!
primitivePragmaSelectors
	"Answer the selectors of pragmas that specify VM primitives.
	 Needed for compile and decomple."
	^#(primitive:
		primitive:error:
		primitive:error:module:
		primitive:module:
		primitive:module:error:
		), CSLayeredMethod pragmaKeyword! !

!Parser methodsFor: '*ContextS2' stamp: 'ag 12/20/2011 21:40'!
layer: aCollectionOfLayers
	^ 0! !

!Symbol methodsFor: '*ContextS2' stamp: 'ag 12/22/2011 15:26'!
withLayerDo: aBlock
	
	^ (Array with: self) withLayersDo: aBlock! !

!Symbol methodsFor: '*ContextS2' stamp: 'ag 12/22/2011 15:26'!
withoutLayerDo: aBlock
	
	^ (Array with: self) withoutLayersDo: aBlock! !

Browser subclass: #CSContextBrowser
	instanceVariableNames: 'selectedLayerIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2'!

CSContextBrowser class
	instanceVariableNames: 'selectedPartialMethods'!

CSContextBrowser class
	instanceVariableNames: 'selectedPartialMethods'!

!CSContextBrowser class methodsFor: 'accessing' stamp: 'ag 12/21/2011 00:06'!
resetSelectedPartialMethods
	
	selectedPartialMethods := nil.! !

!CSContextBrowser class methodsFor: 'accessing' stamp: 'ag 1/10/2012 15:17'!
selectedPartialMethods
	"Globally memorize which partial method has been selected for which layered method in the CSContextBrowser right-most pane.
	Keys are weak to let the partial-method object be garbace collected when they are deletec."
	
	^ selectedPartialMethods ifNil: [ selectedPartialMethods := WeakKeyDictionary new ]! !

!CSContextBrowser methodsFor: 'ui' stamp: 'ag 1/12/2012 12:45'!
buildContextLayerListWith: builder
	
	^ builder pluggableListSpec new
		model: self;
		list: #contextLayerList;
		getIndex: #selectedLayerIndex;
		setIndex: #selectedLayerIndex:;
		menu: #contextLayerMenu:;
		keyPress: #arrowKey:from:;
		yourself! !

!CSContextBrowser methodsFor: 'ui-obsolete' stamp: 'ag 1/12/2012 12:35'!
buildMethodSwitchesWith: builder
	"Build the instance/comment/class switch"
	| panelSpec buttonSpec |
	panelSpec := builder pluggablePanelSpec new.
	panelSpec children: OrderedCollection new.

	buttonSpec := builder pluggableButtonSpec new.
	buttonSpec 
			model: self;
			label: 'base'; 
			enabled: #isLayeredMethodWithBase; 
			action: #showBaseMethod;
			frame: (0@0 corner: 0.4@1).
	panelSpec children addLast: buttonSpec.

	buttonSpec := builder pluggableButtonSpec new.
	buttonSpec 
			model: self;
			label: 'partial'; 
			enabled: #isLayeredMethod; 
			action: #showPartialMethod;
			frame: (0.4@0 corner: 0.8@1).
	panelSpec children addLast: buttonSpec.

	buttonSpec := builder pluggableButtonSpec new.
	buttonSpec 
			model: self;
			label: 'delete'; 
			enabled: #isLayeredMethod; 
			action: #removePartialMethod;
			frame: (0.8@0 corner: 1@1).
	panelSpec children addLast: buttonSpec.

	^ panelSpec! !

!CSContextBrowser methodsFor: 'ui-obsolete' stamp: 'ag 12/21/2011 15:47'!
choosePartialMethod
	
	^ self choosePartialMethod: 'Choose partial method'! !

!CSContextBrowser methodsFor: 'ui-obsolete' stamp: 'ag 12/21/2011 15:49'!
choosePartialMethod: aString
	
	| methods |
	self isLayeredMethod ifFalse: [ ^ nil ].
	methods := self methodReference compiledMethod allPartialMethods.
	^ UIManager default
		chooseFrom: (methods collect: #asMenuString)
		values: methods
		lines: #()
		title: aString! !

!CSContextBrowser methodsFor: 'ui' stamp: 'ag 1/12/2012 13:14'!
classListIndex: aNumber
	
	super classListIndex: aNumber.
	self selectedLayerIndex: 0.! !

!CSContextBrowser methodsFor: 'ui' stamp: 'ag 1/12/2012 13:16'!
contextLayerList
	
	^ classListIndex = 0
		ifTrue: [ Array new ] "No class selected"
		ifFalse: [ 
			{ #'-- all layers --'. #'-- base methods --' },
			self selectedClassOrMetaClass allInvolvedLayers asArray ]! !

!CSContextBrowser methodsFor: 'ui' stamp: 'ag 1/12/2012 12:42'!
contextLayerMenu: aMenu
	
	(self menuHook: aMenu named: #messageCategoryMenu shifted: false) ifTrue: [ ^ aMenu ].
	^ aMenu labels: 'delete layer'
		lines: #(1 1)
		selections: #(deleteSelectedLayer)! !

!CSContextBrowser methodsFor: 'initialize-release' stamp: 'ag 1/12/2012 12:06'!
defaultBrowserTitle
	
	^ 'Context Browser'! !

!CSContextBrowser methodsFor: 'ui' stamp: 'ag 1/12/2012 13:07'!
deleteSelectedLayer
	"When the layer is deleted, we delete all partial methods on it.
	If the partial method was the last one in its LayeredMethod, we delete the LayeredMethod <??>"
	
	! !

!CSContextBrowser methodsFor: 'initialize-release' stamp: 'ag 1/12/2012 13:02'!
initialize
	
	super initialize.
	selectedLayerIndex := 1.! !

!CSContextBrowser methodsFor: 'ui' stamp: 'ag 12/21/2011 15:52'!
isLayeredMethod
	
	^ self layeredMethod notNil! !

!CSContextBrowser methodsFor: 'ui' stamp: 'ag 12/21/2011 15:52'!
isLayeredMethodWithBase
	
	^ self isLayeredMethod and: [ self layeredMethod includesBaseMethod ]! !

!CSContextBrowser methodsFor: 'ui' stamp: 'ag 12/21/2011 15:52'!
layeredMethod
	
	^ (self methodReference notNil and: [ self methodReference compiledMethod class == CSLayeredMethod ])
		ifTrue: [ self methodReference compiledMethod ]! !

!CSContextBrowser methodsFor: 'initialize-release' stamp: 'ag 1/12/2012 13:12'!
openEditString: aString
	"Create a pluggable version of all the views for a Browser, including views and controllers."
	"CSContextBrowser fullOnClass: Browser"
	
	| builder max |
	builder := ToolBuilder default.
	max := self wantsOptionalButtons ifTrue: [ 0.42 ] ifFalse: [ 0.5 ].
	^ self buildWindowWith: builder specs: {
		(0@0 corner: 0.2@max) -> [self buildSystemCategoryListWith: builder].
		(0.2@0 corner: 0.4@(max-0.08)) -> [self buildClassListWith: builder].
		(0.2@(max-0.08) corner: 0.4@max) -> [self buildSwitchesWith: builder].
		(0.6@0 corner: 0.8@max) -> [self buildMessageCategoryListWith: builder].
		(0.4@0 corner: 0.6@max) -> [self buildContextLayerListWith: builder].
		(0.8@0 corner: 1@max) -> [self buildMessageListWith: builder].
		(0@max corner: 1@1) -> [self buildCodePaneWith: builder]
	}! !

!CSContextBrowser methodsFor: 'ui' stamp: 'ag 12/21/2011 15:46'!
refreshCodePane
	
	contents := nil.
	self contentsChanged.! !

!CSContextBrowser methodsFor: 'ui-obsolete' stamp: 'ag 12/21/2011 15:50'!
removePartialMethod
	
	(self choosePartialMethod: 'Choose partial method to delete')
		ifNotNilDo: [ :partialMethod |
			self layeredMethod partialMethods size = 1
				ifTrue: [^ self removeMessage].
			self layeredMethod removePartialMethod: partialMethod.
			self class selectedPartialMethods removeKey: self layeredMethod ifAbsent: [ "nothing" ].
			self refreshCodePane ].! !

!CSContextBrowser methodsFor: 'accessing' stamp: 'ag 1/12/2012 12:45'!
selectedLayerIndex
	^ selectedLayerIndex! !

!CSContextBrowser methodsFor: 'accessing' stamp: 'ag 1/12/2012 13:12'!
selectedLayerIndex: anObject
	
	selectedLayerIndex := anObject.
	self changed: #selectedLayerIndex.
	self changed: #contextLayerList.
	self messageCategoryListIndex: 0. "Deselect categories and messages."
	
	"TODO somehow filter the categories and messages."! !

!CSContextBrowser methodsFor: 'ui-obsolete' stamp: 'ag 12/21/2011 15:53'!
showBaseMethod
	
	self isLayeredMethod
		ifTrue: [
			self class selectedPartialMethods at: self layeredMethod put: self layeredMethod baseMethod.
			self refreshCodePane ].! !

!CSContextBrowser methodsFor: 'ui-obsolete' stamp: 'ag 12/21/2011 15:53'!
showPartialMethod
	
	self choosePartialMethod
		ifNotNilDo: [ :selectedMethod |
			self class selectedPartialMethods at: self layeredMethod put: selectedMethod.
			self refreshCodePane ].! !

!Process methodsFor: '*ContextS2' stamp: 'ag 12/20/2011 21:41'!
csInfo
	"Only the contents of csInfo are accessed, thus no need for a setter in Process"
	
	^ csInfo ifNil: [ csInfo := CSProcessContextInformation new ]! !

!Behavior methodsFor: '*ContextS2' stamp: 'ag 12/11/2011 16:43'!
cs2FileOut
	
	self cs2FileOutOn: (FileStream forceNewFileNamed: self name asString, '.st').! !

!Behavior methodsFor: '*ContextS2' stamp: 'ag 12/11/2011 09:59'!
cs2FileOutCompiledMethod: compiledMethod called: aSelector of: aClass on: aStream
	
	self
		cs2FileOutMultiPart: 
			{ 
			aClass name, ' methodsFor: ''', (aClass organization categoryOfElement: aSelector), ''''.
			compiledMethod getSource
			}
		on: aStream.! !

!Behavior methodsFor: '*ContextS2' stamp: 'ag 12/20/2011 20:33'!
cs2FileOutMethod: meth called: aSelector of: aClass on: stream
	
	(meth isKindOf: CSLayeredMethod)
		ifTrue: [ meth partialMethods do: [ :partialMethod | self cs2FileOutCompiledMethod: partialMethod method called: aSelector of: aClass on: stream ] ]
		ifFalse: [ self cs2FileOutCompiledMethod: meth called: aSelector of: aClass on: stream ]! !

!Behavior methodsFor: '*ContextS2' stamp: 'ag 12/11/2011 09:57'!
cs2FileOutMultiPart: aCollectionOfStrings on: stream
	
	stream nextPut: $!!; cr.
	aCollectionOfStrings
		do: [ :str | self cs2FileOutPart: str on: stream ].
	stream nextPut: $!!; cr.! !

!Behavior methodsFor: '*ContextS2' stamp: 'ag 12/11/2011 10:07'!
cs2FileOutOn: stream
	
	self cs2FileOutPart: self definition on: stream.
	self methodDict keysAndValuesDo: [ :selector :meth |
		self cs2FileOutMethod: meth called: selector of: self on: stream ].! !

!Behavior methodsFor: '*ContextS2' stamp: 'ag 12/11/2011 09:56'!
cs2FileOutPart: aString on: stream
	
	| tokens |
	tokens := aString asString findTokens: '!!' keep: '!!'.
	tokens do: [ :str |
		stream nextPutAll: 
			(str = '!!'
				ifTrue: [ '!!!!' ]
				ifFalse: [ str ]) ].
	stream nextPut: $!!; cr; cr.! !

TestCase subclass: #CSBehavioralVariationsTest
	instanceVariableNames: 'a'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2-Tests'!

!CSBehavioralVariationsTest methodsFor: 'accessing' stamp: 'mpe 11/7/2010 18:30'!
a
	"Answer the value of a"

	^ a! !

!CSBehavioralVariationsTest methodsFor: 'accessing' stamp: 'mpe 11/7/2010 18:30'!
a: anObject
	"Set the value of a"

	a := anObject! !

!CSBehavioralVariationsTest methodsFor: 'running' stamp: 'ag 12/21/2011 20:58'!
layers: someLayers shouldResultIn: anArrayOfSymbols
	
	someLayers withLayersDo: [ self a myMethod ].
	self assert: self a executedMethods size = anArrayOfSymbols size.
	self assert: self a executedMethodsArray = anArrayOfSymbols.! !

!CSBehavioralVariationsTest methodsFor: 'running' stamp: 'mpe 11/7/2010 18:30'!
setUp

	self a: CSClassA new.! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 16:20'!
test01ExecuteBaseMethod
	
	self
		layers: #()
		shouldResultIn: #(base).! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 16:05'!
test02ExecuteBeforeBaseMethod
	
	self
		layers: #(before)
		shouldResultIn: #(before base).! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 16:06'!
test04aExecuteBeforeBaseAfterMethod
	
	self
		layers: #(before after)
		shouldResultIn: #(before base after).! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 16:06'!
test04bExecuteBeforeBaseAfterMethod
	
	self
		layers: #(after before)
		shouldResultIn: #(before base after).! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 16:06'!
test05ExecuteAroundMethod
	
	self
		layers: #(replace)
		shouldResultIn: #(replace).! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 16:06'!
test06ExecuteAroundProceedBaseMethod
	
	self
		layers: #(proceed1)
		shouldResultIn: #(proceed1before base proceed1after).! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 16:07'!
test07aExecuteAroundProceedAroundMethod
	
	self
		layers: #(proceed1 replace)
		shouldResultIn: #(proceed1before replace proceed1after).! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 20:40'!
test07bExecuteAroundProceedAroundMethod
	
	self
		layers: #(replace proceed1)
		shouldResultIn: #(replace).! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 16:07'!
test08ExecuteBeforeAroundAfterMethod
	
	self
		layers: #(before replace after)
		shouldResultIn: #(before replace).! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 20:57'!
test09ExecuteBeforeAroundProceedBaseAfterMethod
	
	self
		layers: #(before proceed1 after)
		shouldResultIn: #(before proceed1before base after proceed1after).! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 20:42'!
test09aExecuteBeforeAroundProceedBaseAfterMethod
	
	self
		layers: #(proceed1 before after)
		shouldResultIn: #(proceed1before before base after proceed1after).! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 16:11'!
test10ExecuteBeforeAroundProceedAroundAfterMethod
	
	self
		layers: #(before replace proceed1 after)
		shouldResultIn: #(before replace).! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 16:11'!
test10aExecuteBeforeAroundProceedAroundAfterMethod
	
	self
		layers: #(after before replace proceed1)
		shouldResultIn: #(before replace after).! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 20:57'!
test11aReturnValueFromAround
	
	#replacedReturn withLayerDo: [ self assert: self a myReturnMethod = #replacedReturn ].! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 12:01'!
test11bReturnValueFromBase
	
	self assert: self a myReturnMethod = #base.! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 16:13'!
test12MultipleChainedProceeds
	
	self
		layers: #(proceed1 proceed2)
		shouldResultIn: #(proceed1before proceed2before base proceed2after proceed1after).! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 20:58'!
test13MultipleChainedProceedsWithoutBase
	
	self
		layers: #(proceed1 proceed2 replace)
		shouldResultIn: #(proceed1before proceed2before replace proceed2after proceed1after).! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/21/2011 16:14'!
test13aMultipleChainedProceedsWithoutBase
	
	self
		layers: #(replace proceed1 proceed2)
		shouldResultIn: #(replace).! !

!CSBehavioralVariationsTest methodsFor: 'testing' stamp: 'ag 12/23/2011 16:57'!
test14MultiArgumentProceed
	
	#(someLayer someLayer2) withLayersDo: [ self assert: 19 = (CSClassA new myMethod: 12 and: 5) ].! !

TestCase subclass: #CSContextSTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2-Tests'!

CSContextSTest subclass: #CSAddingMethodsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2-Tests'!

!CSAddingMethodsTest methodsFor: 'testing' stamp: 'ag 12/22/2011 11:55'!
addMethodFrom: code
	
	self addMethodFrom: code toClass: CSClassA.! !

!CSAddingMethodsTest methodsFor: 'testing' stamp: 'ag 12/22/2011 11:55'!
test01AddCompiledMethodExistingNil
	
	self addMethodFrom: self baseCode.
	self assert: self generatedMethod class == CompiledMethod.! !

!CSAddingMethodsTest methodsFor: 'testing' stamp: 'ag 12/22/2011 11:55'!
test02AddCompiledMethodExistingCompiledMethod
	
	| compiledMethod |
	self addMethodFrom: self baseCode.
	compiledMethod := self generatedMethod.
	self addMethodFrom: self baseCode.
	self deny: self generatedMethod == compiledMethod.! !

!CSAddingMethodsTest methodsFor: 'testing' stamp: 'ag 12/22/2011 11:57'!
test03aAddCompiledMethodExistingLayeredMethodWithoutBase
	
	| layeredMethod |
	self addMethodFrom: self singleLayerCode.
	layeredMethod := self generatedMethod.
	self deny: layeredMethod includesBaseMethod.
	self assert: layeredMethod baseMethod isNil.
	
	self addMethodFrom: self baseCode.
	self assert: layeredMethod includesBaseMethod.
	self assert: layeredMethod partialMethods size == 1.
	self assert: layeredMethod partialMethods first class == CSPartialMethod.
	self assert: layeredMethod baseMethod class == CompiledMethod.! !

!CSAddingMethodsTest methodsFor: 'testing' stamp: 'ag 12/22/2011 12:04'!
test03bAddCompiledMethodExistingLayeredMethodWithBase
	
	| layeredMethod oldBaseMethod |
	self addMethodFrom: self baseCode.
	self addMethodFrom: self singleLayerCode.
	layeredMethod := self generatedMethod.
	self assert: layeredMethod partialMethods size = 1.
	oldBaseMethod := layeredMethod baseMethod.
	
	self addMethodFrom: self baseCode.
	self assert: layeredMethod partialMethods size = 1.
	self deny: layeredMethod baseMethod == oldBaseMethod.! !

!CSAddingMethodsTest methodsFor: 'testing' stamp: 'ag 12/22/2011 11:58'!
test04AddPartialMethodExistingNil
	
	| layeredMethod |
	self addMethodFrom: self singleLayerCode.
	layeredMethod := self generatedMethod.
	self assert: layeredMethod class == CSLayeredMethod.
	self deny: layeredMethod includesBaseMethod.
	self assert: layeredMethod partialMethods size = 1.! !

!CSAddingMethodsTest methodsFor: 'testing' stamp: 'ag 12/22/2011 12:00'!
test05AddPartialMethodExistingCompiledMethod
	
	| compiledMethod layeredMethod |
	self addMethodFrom: self baseCode.
	compiledMethod := self generatedMethod.
	self assert: compiledMethod class == CompiledMethod.
	
	self addMethodFrom: self singleLayerCode.
	layeredMethod := self generatedMethod.
	self assert: layeredMethod class == CSLayeredMethod.
	self assert: layeredMethod partialMethods size = 1.
	self assert: layeredMethod baseMethod method == compiledMethod.
	self assert: layeredMethod partialMethods first class == CSPartialMethod.
	self assert: layeredMethod baseMethod == compiledMethod.! !

!CSAddingMethodsTest methodsFor: 'testing' stamp: 'ag 12/23/2011 11:31'!
test06aAddPartialMethodExistingLayeredMethodLayerCombinationNotExistingWithBase
	
	| layeredMethod oldPartial |
	self addMethodFrom: self baseCode.
	self addMethodFrom: self singleLayerCode.
	layeredMethod := self generatedMethod.
	self assert: layeredMethod partialMethods size = 1.
	self assert: layeredMethod includesBaseMethod.
	oldPartial := layeredMethod partialMethods first.
	
	self addMethodFrom: self threeLayersCode.
	self assert: layeredMethod partialMethods size = 2.
	self assert: layeredMethod partialMethods first layers = #(l1).
	self assert: layeredMethod partialMethods second layers = #(l1 l2 l3).
	self assert: layeredMethod partialMethods first == oldPartial.! !

!CSAddingMethodsTest methodsFor: 'testing' stamp: 'ag 12/23/2011 11:31'!
test06bAddPartialMethodExistingLayeredMethodLayerCombinationNotExistingWithoutBase
	
	| layeredMethod |
	self addMethodFrom: self singleLayerCode.
	layeredMethod := self generatedMethod.
	self assert: layeredMethod partialMethods size = 1.
	self deny: layeredMethod includesBaseMethod.
	
	self addMethodFrom: self threeLayersCode.
	self assert: layeredMethod partialMethods size = 2.
	self assert: layeredMethod partialMethods first layers = #(l1).
	self assert: layeredMethod partialMethods second layers = #(l1 l2 l3).
	self deny: layeredMethod includesBaseMethod.! !

!CSAddingMethodsTest methodsFor: 'testing' stamp: 'ag 12/23/2011 11:31'!
test06cAddPartialMethodExistingLayeredMethodLayerCombinationExistingSingle
	
	| layeredMethod oldPartialMethod |
	self addMethodFrom: self baseCode.
	self addMethodFrom: self singleLayerCode.
	layeredMethod := self generatedMethod.
	self assert: layeredMethod partialMethods size = 1.
	oldPartialMethod := layeredMethod partialMethods first.
	
	self addMethodFrom: self singleLayerCode.
	self assert: layeredMethod partialMethods size = 1.
	self assert: layeredMethod partialMethods first layers = #(l1).
	self deny: layeredMethod partialMethods first == oldPartialMethod.! !

!CSAddingMethodsTest methodsFor: 'testing' stamp: 'ag 12/23/2011 11:31'!
test06dAddPartialMethodExistingLayeredMethodLayerCombinationExistingMultiple
	
	| layeredMethod  oldBaseMethod oldPartialMethod |
	self addMethodFrom: self baseCode.
	self addMethodFrom: self threeLayersCode.
	layeredMethod := self generatedMethod.
	self assert: layeredMethod partialMethods size = 1.
	oldPartialMethod := layeredMethod partialMethods first.
	oldBaseMethod := layeredMethod baseMethod.
	
	self addMethodFrom: self threeLayersCode.
	self assert: layeredMethod partialMethods size = 1.
	self assert: layeredMethod partialMethods first layers = #(l1 l2 l3).
	self deny: layeredMethod partialMethods first == oldPartialMethod.
	self assert: layeredMethod baseMethod == oldBaseMethod.
	oldPartialMethod := layeredMethod partialMethods first.
	
	self addMethodFrom: self baseCode.
	self deny: layeredMethod baseMethod == oldBaseMethod.
	self assert: layeredMethod partialMethods first == oldPartialMethod.! !

CSContextSTest subclass: #CSCompilerExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2-Tests'!

!CSCompilerExtensionTest methodsFor: 'testing' stamp: 'ag 12/22/2011 13:43'!
compile: aSourceCodeString
	"This does not add the method to the class."
	
	| method |
	method := (CSClassA
		compile: aSourceCodeString
		classified: 'generated partial methods'
		notifying: nil
		trailer: CompiledMethodTrailer empty
		ifFail: nil	) method.
	self deny: (CSClassA methodDict includes: self generatedSelector).
	^ method! !

!CSCompilerExtensionTest methodsFor: 'testing' stamp: 'ag 12/22/2011 13:44'!
test01CompileBaseMethod
	
	self assert: (self compile: self baseCode) class = CompiledMethod.! !

!CSCompilerExtensionTest methodsFor: 'testing' stamp: 'ag 12/23/2011 11:31'!
test02CompileBaseMethodAsPartialMethod
	
	| method |
	method := (self compile: self baseCode) asPartialMethod.
	self assert: method class = CSPartialMethod.
	self assert: method layers isNil.
	self assert: method method class == CompiledMethod.
	self assert: method method = (self compile: self baseCode).! !

!CSCompilerExtensionTest methodsFor: 'testing' stamp: 'ag 12/23/2011 11:31'!
test03aCompileMultipleLayers
	
	| method |
	method := self compile: self singleLayerCode.
	self assert: method class = CSPartialMethod.
	self assert: method layers = #(l1).
	self assert: method method class == CompiledMethod.! !

!CSCompilerExtensionTest methodsFor: 'testing' stamp: 'ag 12/23/2011 11:31'!
test03bCompileMultipleLayers
	
	| method |
	method := self compile: self threeLayersCode.
	self assert: method class = CSPartialMethod.
	self assert: method layers = #(l1 l2 l3).
	self assert: method method class == CompiledMethod.! !

!CSContextSTest class methodsFor: 'constants' stamp: 'ag 12/21/2011 21:35'!
generatedSelector

	^ #generatedPartialMethod! !

!CSContextSTest methodsFor: 'running' stamp: 'ag 12/22/2011 11:53'!
addMethodFrom: aSourceCodeString toClass: aClass
	
	aClass
		compile: aSourceCodeString
		classified: 'partial methods generated during tests'
		withStamp: Utilities changeStamp
		notifying: nil.
	self assert: (aClass methodDict includesKey: self generatedSelector).! !

!CSContextSTest methodsFor: 'method code' stamp: 'ag 12/21/2011 21:40'!
baseCode
	
	^ self codeForLayers: nil! !

!CSContextSTest methodsFor: 'method code' stamp: 'ag 12/22/2011 13:47'!
codeForLayers: layerSymbols
	"Method code contains only"
	
	^ self codeForLayers: layerSymbols withContent: ''! !

!CSContextSTest methodsFor: 'method code' stamp: 'ag 12/22/2011 13:47'!
codeForLayers: layerSymbols withContent: contentString
	"Method code contains only"
	
	^ (self generatedSelector, 
		'\"This method has been generated by ', self class name, 
		' and may be deleted."\',
		(layerSymbols ifNil: [ '' ] ifNotNil: [ '<layer: ', layerSymbols printString, '>' ]),
		contentString)
			withCRs! !

!CSContextSTest methodsFor: 'running' stamp: 'ag 12/22/2011 11:42'!
deleteMethod
	
	CSClassA removeSelector: self generatedSelector.! !

!CSContextSTest methodsFor: 'accessing' stamp: 'ag 12/22/2011 11:54'!
generatedMethod
	
	^ CSClassA >> self generatedSelector! !

!CSContextSTest methodsFor: 'accessing' stamp: 'ag 12/21/2011 21:35'!
generatedSelector

	^ self class generatedSelector! !

!CSContextSTest methodsFor: 'running' stamp: 'ag 12/22/2011 11:42'!
setUp
	
	self
		deleteMethod;
		deny: (CSClassA methodDict includesKey: self generatedSelector).! !

!CSContextSTest methodsFor: 'method code' stamp: 'ag 12/22/2011 11:51'!
singleLayerCode
	
	^ self codeForLayers: #l1! !

!CSContextSTest methodsFor: 'running' stamp: 'ag 12/22/2011 11:42'!
tearDown
	
	self deleteMethod.! !

!CSContextSTest methodsFor: 'method code' stamp: 'ag 12/22/2011 11:51'!
threeLayersCode
	
	^ self codeForLayers: #(l1 l2 l3)! !

!CSContextSTest methodsFor: 'method code' stamp: 'ag 12/22/2011 11:52'!
twoLayersCode
	
	^ self codeForLayers: #(l1 l2)! !

CSContextSTest subclass: #CSPartialMethodTest
	instanceVariableNames: 'partialMethod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2-Tests'!

!CSPartialMethodTest methodsFor: 'running' stamp: 'ag 12/22/2011 13:50'!
deleteMethod
	
	super deleteMethod.
	CSClassB removeSelector: self generatedSelector.
	CSSubclassB removeSelector: self generatedSelector.! !

!CSPartialMethodTest methodsFor: 'method code' stamp: 'ag 12/22/2011 13:48'!
partialCountMethod
	
	^ self codeForLayers: #l1 withContent: '^ count + 1'! !

!CSPartialMethodTest methodsFor: 'running' stamp: 'ag 12/22/2011 13:59'!
partialMethodFrom: aClass
	
	^ (aClass >> self generatedSelector) partialMethods first! !

!CSPartialMethodTest methodsFor: 'method code' stamp: 'ag 12/22/2011 13:49'!
partialSubCountMethod
	
	^ self codeForLayers: #l1 withContent: '^ super generatedPartialMethod + count'! !

!CSPartialMethodTest methodsFor: 'method code' stamp: 'ag 12/22/2011 13:49'!
partialSubSuperCountMethod
	
	^ self codeForLayers: #l1 withContent: '^ super count'! !

!CSPartialMethodTest methodsFor: 'running' stamp: 'ag 12/22/2011 13:50'!
setUp
	
	super setUp.
	self addMethodFrom: self partialCountMethod toClass: CSClassB.! !

!CSPartialMethodTest methodsFor: 'testing' stamp: 'ag 12/23/2011 11:31'!
test01CompilePartialMethodAndToClass
	
	| partialMethod |
	partialMethod := self partialMethodFrom: CSClassB.
	self assert: partialMethod layers = #(l1).
	self assert: partialMethod method class == CompiledMethod.
	self assert: partialMethod method selector == self generatedSelector.
	self assert: partialMethod method methodClass == CSClassB.! !

!CSPartialMethodTest methodsFor: 'testing' stamp: 'ag 12/22/2011 13:58'!
test02aCallPartialMethod
	
	self assert: ((self partialMethodFrom: CSClassB) valueWithReceiver: CSClassB new arguments: #()) = 2.! !

!CSPartialMethodTest methodsFor: 'testing' stamp: 'ag 12/22/2011 14:04'!
test02bCallPartialMethodFromInstance
	
	#l1 withLayerDo: [ self assert: CSClassB new generatedPartialMethod = 2 ].! !

!CSPartialMethodTest methodsFor: 'testing' stamp: 'ag 12/22/2011 14:00'!
test03CallPartialMethodWithSuperCall
	
	CSClassB methodDict at: self generatedSelector put: (self partialMethodFrom: CSClassB).
	self addMethodFrom: self partialSubCountMethod toClass: CSSubclassB.
	CSSubclassB methodDict at: self generatedSelector put: (self partialMethodFrom: CSSubclassB).
	self assert: CSSubclassB new generatedPartialMethod = 3.! !

!CSPartialMethodTest methodsFor: 'testing' stamp: 'ag 12/22/2011 14:05'!
test03aCallPartialMethodWithSuperCall
	
	self addMethodFrom: self partialSubCountMethod toClass: CSSubclassB.
	#l1 withLayerDo: [ self assert: CSSubclassB new generatedPartialMethod = 3 ].! !

!CSPartialMethodTest methodsFor: 'testing' stamp: 'ag 12/22/2011 14:05'!
test04CallPartialMethodWithSuperCallToStandardMethod
	
	CSClassB methodDict at: self generatedSelector put: (self partialMethodFrom: CSClassB).
	self addMethodFrom: self partialSubSuperCountMethod toClass: CSSubclassB.
	CSSubclassB methodDict at: self generatedSelector put: (self partialMethodFrom: CSSubclassB).
	self assert: CSSubclassB new generatedPartialMethod = 1.! !

!CSPartialMethodTest methodsFor: 'testing' stamp: 'ag 12/22/2011 14:00'!
test05CallPartialMethodFromSubclass

	CSClassB methodDict at: self generatedSelector put: (self partialMethodFrom: CSClassB).
	self assert: CSSubclassB new generatedPartialMethod = 2.! !

!CSPartialMethodTest methodsFor: 'testing' stamp: 'ag 12/22/2011 14:06'!
test05aCallPartialMethodFromSubclass
	
	#l1 withLayerDo: [ self assert: CSSubclassB new generatedPartialMethod = 2 ].! !

TestCase subclass: #CSLayerInstancesTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2-Tests'!

!CSLayerInstancesTest methodsFor: 'constants' stamp: 'ag 12/22/2011 17:14'!
layerName
	
	^ #testlayer! !

!CSLayerInstancesTest methodsFor: 'testing-current-layer' stamp: 'ag 12/23/2011 11:34'!
test1NoCurrentLayer
	
	self assert: self currentLayers isNil.! !

!CSLayerInstancesTest methodsFor: 'testing-current-layer' stamp: 'ag 12/23/2011 11:34'!
test2NestedLayersNoCurrentLayer
	
	#a withLayerDo: [
		#b withLayerDo: [
			self assert: self currentLayers isNil ] ].! !

!CSLayerInstancesTest methodsFor: 'testing-current-layer' stamp: 'ag 12/23/2011 11:34'!
test3SingleLayerNoCurrentLayer
	
	#a withLayerDo: [ self assert: self currentLayers isNil ].! !

!CSLayerInstancesTest methodsFor: 'testing-current-layer' stamp: 'ag 12/23/2011 11:22'!
test4CurrentLayerWithoutPartialMethod
	
	#zzz withLayerDo: [ self assert: CSClassA new getCurrentLayerA isNil ].! !

!CSLayerInstancesTest methodsFor: 'testing-current-layer' stamp: 'ag 12/23/2011 11:36'!
test5CurrentLayerSingleLayer
	
	| layer |
	#a withLayerDo: [ layer := CSClassA new getCurrentLayerA ].
	self assert: layer notNil.
	self assert: layer size == 1.
	self assert: layer first layerName == #a.! !

!CSLayerInstancesTest methodsFor: 'testing-current-layer' stamp: 'ag 12/23/2011 11:39'!
test5aCurrentLayerSingleLayer
	
	| layer |
	#(b a) withLayersDo: [ layer := CSClassA new getCurrentLayerA ].
	self assert: layer notNil.
	self assert: layer size == 1.
	self assert: layer first layerName == #a.! !

!CSLayerInstancesTest methodsFor: 'testing-current-layer' stamp: 'ag 12/23/2011 11:36'!
test6CurrentLayerSingleLayerTwoLayerMethod
	
	| layer |
	#a withLayerDo: [ layer := CSClassA new getCurrentLayerAB ].
	self assert: layer notNil.
	self assert: layer size == 1.
	self assert: layer first layerName == #a.! !

!CSLayerInstancesTest methodsFor: 'testing-current-layer' stamp: 'ag 12/23/2011 11:36'!
test7CurrentLayerSingleLayerTwoLayerMethod2
	
	| layer |
	#b withLayerDo: [ layer := CSClassA new getCurrentLayerAB ].
	self assert: layer notNil.
	self assert: layer size == 1.
	self assert: layer first layerName == #b.! !

!CSLayerInstancesTest methodsFor: 'testing-current-layer' stamp: 'ag 12/23/2011 11:37'!
test8CurrentLayerMultipleLayersTwoLayerMethod
	
	| layer |
	#(a b) withLayersDo: [ layer := CSClassA new getCurrentLayerAB ].
	self assert: layer notNil.
	self assert: layer size == 2.
	self assert: (layer collect: #layerName) = #(a b).! !

!CSLayerInstancesTest methodsFor: 'testing-current-layer' stamp: 'ag 12/23/2011 11:38'!
test9CurrentLayerMultipleLayersTwoLayerMethod2
	
	| layer |
	#(a b) withLayersDo: [ layer := CSClassA new getCurrentLayerBA ].
	self assert: layer notNil.
	self assert: layer size == 2.
	self assert: (layer collect: #layerName) = #(a b).! !

!CSLayerInstancesTest methodsFor: 'testing' stamp: 'ag 12/22/2011 17:22'!
testAnonymousLayerInstance
	
	| instance |
	CSLayer layerNamed: self layerName.
	self assert: (Smalltalk includesKey: self layerName).
	CSLayer resetLayer: self layerName.
	self assert: (Smalltalk includesKey: self layerName) not.
	instance := CSLayer layerNamed: self layerName.
	self assert: (Smalltalk at: self layerName) == instance.! !

!CSLayerInstancesTest methodsFor: 'testing' stamp: 'ag 12/22/2011 17:20'!
testAnonymousLayerInstanceReplaced
	
	| instance |
	CSLayer resetLayer: self layerName.
	instance := CSLayer layerNamed: self layerName.
	CSLayer resetLayer: self layerName.
	self deny: (CSLayer layerNamed: self layerName) == instance.! !

!CSLayerInstancesTest methodsFor: 'testing' stamp: 'ag 12/22/2011 17:21'!
testAnonymousLayerInstanceSame
	
	| instance |
	CSLayer resetLayer: self layerName.
	instance := CSLayer layerNamed: self layerName.
	self assert: (CSLayer layerNamed: self layerName) == instance.! !

!CSLayerInstancesTest methodsFor: 'testing' stamp: 'ag 12/22/2011 17:22'!
testSubclassedLayerInstance
	"This tests 2 things: #resetInstance (removes the global variable) and #instance (creates it and adds it as global variable)"
	
	| instance |
	CSTestLayer instance.
	self assert: (Smalltalk includesKey: CSTestLayer globalLayerName).
	CSTestLayer resetInstance.
	self assert: (Smalltalk includesKey: CSTestLayer globalLayerName) not.
	instance := CSTestLayer instance.
	self assert: (Smalltalk at: CSTestLayer globalLayerName) == instance.! !

!CSLayerInstancesTest methodsFor: 'testing' stamp: 'ag 12/23/2011 12:00'!
testSubclassedLayerInstanceAccessableWithLayerNamed
	
	| instance |
	instance := CSTestLayer instance.
	self assert: instance == (CSLayer layerNamed: CSTestLayer globalLayerName).
	CSTestLayer resetInstance.
	instance := CSTestLayer instance.
	self assert: instance == (CSLayer layerNamed: CSTestLayer globalLayerName).! !

!CSLayerInstancesTest methodsFor: 'testing' stamp: 'ag 12/22/2011 17:22'!
testSubclassedLayerInstanceReplaced
	
	| instance |
	CSTestLayer resetInstance.
	instance := CSTestLayer instance.
	CSTestLayer resetInstance.
	self deny: CSTestLayer instance == instance.! !

!CSLayerInstancesTest methodsFor: 'testing' stamp: 'ag 12/23/2011 13:53'!
testSubclassedLayerInstanceResettableWithLayerNamed
	
	| instance |
	instance := CSTestLayer instance.
	CSLayer resetLayer: CSTestLayer globalLayerName.
	self deny: instance == CSTestLayer instance.! !

!CSLayerInstancesTest methodsFor: 'testing' stamp: 'ag 12/22/2011 17:21'!
testSubclassedLayerInstanceSame
	
	| instance |
	CSTestLayer resetInstance.
	instance := CSTestLayer instance.
	self assert: CSTestLayer instance == instance.! !

!CSLayerInstancesTest methodsFor: 'testing-current-layer' stamp: 'ag 12/23/2011 11:40'!
testz10CurrentLayerMultipleLayersTwoLayerMethod3
	
	| layer |
	#(b a) withLayersDo: [ layer := CSClassA new getCurrentLayerAB ].
	self assert: layer notNil.
	self assert: layer size == 2.
	self assert: (layer collect: #layerName) = #(b a).! !

!CSLayerInstancesTest methodsFor: 'testing-current-layer' stamp: 'ag 12/23/2011 11:44'!
testz11CurrentLayerNestedCalls
	"First partial method on layer #b is run, and inside another partial method on layer #a."
	
	| layer |
	#b withLayerDo: [ layer := CSClassA new getCurrentLayerOf: #getCurrentLayerA on: #a ].
	self assert: layer notNil.
	self assert: layer size == 1.
	self assert: layer first layerName == #a.! !

!CSLayerInstancesTest methodsFor: 'testing-current-layer' stamp: 'ag 12/23/2011 11:47'!
testz12CurrentLayerNestedCalls2
	"Current Layer not changed when another partial method is invoked in its base version."
	
	| layer |
	#b withLayerDo: [ layer := CSClassA new getCurrentLayerOf: #getCurrentLayerA on: #zzz ].
	self assert: layer notNil.
	self assert: layer size == 1.
	self assert: layer first layerName == #b.! !

!CSLayerInstancesTest methodsFor: 'testing-current-layer' stamp: 'ag 12/23/2011 11:48'!
testz12CurrentLayerNestedCalls3
	
	| layer |
	#a withLayerDo: [ layer := CSClassA new getCurrentLayerOf: #getCurrentLayerA on: #zzz ].
	self assert: layer notNil.
	self assert: layer size == 1.
	self assert: layer first layerName == #a.! !

!CSLayerInstancesTest methodsFor: 'testing-current-layer' stamp: 'ag 12/23/2011 11:48'!
testz12CurrentLayerNestedCalls4
	
	| layer |
	#b withLayerDo: [ layer := CSClassA new getCurrentLayerOf: #getCurrentLayerAB on: #zzz ].
	self assert: layer notNil.
	self assert: layer size == 1.
	self assert: layer first layerName == #b.! !

!CSLayerInstancesTest methodsFor: 'testing-current-layer' stamp: 'ag 12/23/2011 11:48'!
testz12CurrentLayerNestedCalls5
	
	| layer |
	#b withLayerDo: [ layer := CSClassA new getCurrentLayerOf: #getCurrentLayerBA on: #zzz ].
	self assert: layer notNil.
	self assert: layer size == 1.
	self assert: layer first layerName == #b.! !

TestCase subclass: #CSLayerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2-Tests'!

!CSLayerTest methodsFor: 'accessing' stamp: 'ag 12/17/2011 21:52'!
activeLayers

	^ self process csInfo activeLayers! !

!CSLayerTest methodsFor: 'accessing' stamp: 'mp 6/11/2010 20:54'!
process
	^ Processor activeProcess! !

!CSLayerTest methodsFor: 'layer tests' stamp: 'ag 12/17/2011 21:50'!
test01LayerCollectionInitialization

	self assert: self activeLayers isEmpty.! !

!CSLayerTest methodsFor: 'layer tests' stamp: 'ag 12/17/2011 21:50'!
test02aSingleLayerActivation
	#l1 withLayerDo: [
		self assert: self activeLayers notEmpty.
		self assert: self activeLayers first = #l1.
		].
	self assert: self activeLayers isEmpty.! !

!CSLayerTest methodsFor: 'layer tests' stamp: 'ag 12/17/2011 21:51'!
test02bSingleLayerActivation
	#(l1) withLayersDo: [
		self assert: self activeLayers notEmpty.
		self assert: self activeLayers first = #l1.
		].
	self assert: self activeLayers isEmpty.! !

!CSLayerTest methodsFor: 'layer tests' stamp: 'ag 12/21/2011 11:50'!
test03ComplicatedNestedLayerActivation
	#l1 withLayerDo: [
		self assert: self activeLayers notEmpty.
		self assert: self activeLayers first = #l1.
		#(l1 l2 l3) withLayersDo: [
			self assert: self activeLayers size = 3.
			self assert: self activeLayers first = #l1.
			self assert: self activeLayers last = #l3.
			#l1 withoutLayerDo: [
				self assert: self activeLayers size = 2.
				self assert: self activeLayers first = #l2.
				self assert: self activeLayers last = #l3.
				].
			self assert: self activeLayers size = 3.
			self assert: self activeLayers first = #l1.
			self assert: self activeLayers last = #l3.
			].
		self assert: self activeLayers size = 1.
		self assert: self activeLayers first = 'l1'.
		].
	self assert: self activeLayers isEmpty.! !

!CSLayerTest methodsFor: 'layer tests' stamp: 'ag 12/17/2011 21:51'!
test03NestedLayerActivation
	#l1 withLayerDo: [
		self assert: self activeLayers notEmpty.
		self assert: self activeLayers first = #l1.
		#l1 withLayerDo: [
			self assert: self activeLayers size = 1.
			self assert: self activeLayers first = #l1.
			#l1 withLayerDo: [
				self assert: self activeLayers size = 1.
				self assert: self activeLayers first = #l1.
				]
			].
		self assert: self activeLayers size = 1.
		self assert: self activeLayers first = 'l1'.
		].
	self assert: self activeLayers isEmpty.! !

!CSLayerTest methodsFor: 'layer tests' stamp: 'ag 12/17/2011 21:51'!
test04LayerDeactivation
	#l1 withLayerDo: [
		self assert: self activeLayers size = 1.
		self assert: self activeLayers first = #l1.
		#l1 withoutLayerDo: [
			self assert: self activeLayers isEmpty.
			].
		self assert: self activeLayers size = 1.
		self assert: self activeLayers first = #l1.
		].
	self assert: self activeLayers isEmpty.! !

!CSLayerTest methodsFor: 'layer tests' stamp: 'ag 12/17/2011 21:51'!
test05MultipleLayerDeactivation
	#(l1 l2 l3) withLayersDo: [
		self assert: self activeLayers size = 3.
		self assert: self activeLayers last = #l3.
		#l2 withoutLayerDo: [
			self assert: self activeLayers size = 2.
			self assert: self activeLayers last = #l3.
			self assert: self activeLayers first = #l1].
		self assert: self activeLayers size = 3.
		self assert: self activeLayers second = #l2.
		#(l1 l2) withoutLayersDo: [
			#l3 withoutLayerDo: [self assert: self activeLayers isEmpty].
			self assert: self activeLayers first = #l3].
		self assert: self activeLayers size = 3.
		].
	self assert: self activeLayers isEmpty.! !

!CSLayerTest methodsFor: 'layer tests' stamp: 'ag 12/17/2011 21:51'!
test06LayerDeactivationInNonLocalReturns
	self assert: self activeLayers isEmpty.
	self deny: CSClassA new nonLocalReturn.
	self assert: self activeLayers isEmpty.! !

Object subclass: #CSClassA
	instanceVariableNames: 'executedMethods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2-Tests-Sandbox'!

!CSClassA methodsFor: 'accessing' stamp: ''!
executed: aSymbol

	self executedMethods add: aSymbol.! !

!CSClassA methodsFor: 'accessing' stamp: ''!
executedMethods
	"Answer the value of executedMethods"

	^ executedMethods! !

!CSClassA methodsFor: 'accessing' stamp: ''!
executedMethods: anObject
	"Set the value of executedMethods"

	executedMethods := anObject! !

!CSClassA methodsFor: 'accessing' stamp: ''!
executedMethodsArray

	^ self executedMethods asArray! !

!CSClassA methodsFor: 'partial methods' stamp: 'ag 12/23/2011 11:34'!
getCurrentLayerA
	
	^ self currentLayer! !

!CSClassA methodsFor: 'partial methods' stamp: 'ag 12/23/2011 11:34'!
getCurrentLayerAB
	<layer: #(a b)>
	^ self currentLayer! !

!CSClassA methodsFor: 'partial methods' stamp: 'ag 12/23/2011 11:35'!
getCurrentLayerBA
	<layer: #(b a)>
	^ self currentLayer! !

!CSClassA methodsFor: 'partial methods' stamp: 'ag 12/23/2011 11:18'!
getCurrentLayerOf: aSymbol on: aLayer
	<layer: #(a b)>
	aLayer withLayerDo: [ ^ self perform: aSymbol ].! !

!CSClassA methodsFor: 'initialize-release' stamp: ''!
initialize
	
	self executedMethods: OrderedCollection new.! !

!CSClassA methodsFor: 'partial methods' stamp: 'ag 12/23/2011 16:44'!
myMethod
	
	self executed: #base! !

!CSClassA methodsFor: 'partial methods' stamp: 'ag 12/23/2011 16:55'!
myMethod: arg1 and: arg2
	
	^ arg1 + arg2! !

!CSClassA methodsFor: 'partial methods' stamp: ''!
myReturnMethod
	<layer: #replacedReturn>
	^ #replacedReturn! !

!CSClassA methodsFor: 'non local return' stamp: ''!
nonLocalReturn
	|  aBlock |
	aBlock :=  [^ false].
	#l1 withLayerDo: [self valueBlock: aBlock].! !

!CSClassA methodsFor: 'non local return' stamp: ''!
valueBlock: aBlock

	aBlock value.! !

Object subclass: #CSClassB
	instanceVariableNames: 'count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2-Tests-Sandbox'!

!CSClassB methodsFor: 'accessing' stamp: ''!
count

	^ count! !

!CSClassB methodsFor: 'initialize-release' stamp: ''!
initialize

	count := 1.! !

CSClassB subclass: #CSSubclassB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2-Tests-Sandbox'!

Object subclass: #CSLayer
	instanceVariableNames: 'layerName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2'!

!CSLayer class methodsFor: 'private' stamp: 'ag 12/23/2011 14:22'!
checkIsSubclass
	
	self == CSLayer ifTrue: [ self error: 'Cannot access singleton-instance of CSLayer itself.' ].! !

!CSLayer class methodsFor: 'private' stamp: 'ag 12/23/2011 14:33'!
getGlobalInstanceOf: aClass withName: aSymbol
	
	^ (Smalltalk includesKey: aSymbol)
		ifTrue: [ | layer |
			layer := Smalltalk at: aSymbol.
			(layer isKindOf: CSLayer)
				ifTrue: [
					layer class == aClass
						ifTrue: [ layer ]
						ifFalse: [ "Delete obsolete CSLayer instances. Kind of a side case, but keeps the system clean."
							Smalltalk removeKey: aSymbol.
							self getGlobalInstanceOf: aClass withName: aSymbol ] ]
				ifFalse: [ self error: 'Global ', aSymbol, ' already exists, cannot create layer with that name.' ] ]
		ifFalse: [
			Smalltalk at: aSymbol put: (aClass basicNew initialize: aSymbol; yourself) ] "Avoid #new"! !

!CSLayer class methodsFor: 'subclassed layers' stamp: 'ag 12/22/2011 16:22'!
globalLayerName
	"The name of the layer to be accessed as a global variable. Is accessible as soon as instance has been invoked."
	
	self subclassResponsibility.! !

!CSLayer class methodsFor: 'subclassed layers' stamp: 'ag 12/23/2011 14:29'!
instance
	"Singleton-pattern with registration in the global namespace.
	Only for subclasses of CSLayer, CSLayer itself has multiple 'anonymous' instances (which are also registered globally)"
	
	self checkIsSubclass.
	^ self
		getGlobalInstanceOf: self
		withName: self globalLayerName! !

!CSLayer class methodsFor: 'layers' stamp: 'ag 12/23/2011 14:29'!
layerNamed: aSymbol
	"Return the CSLayer-instance associated with aSymbol.
	If a subclass with the according globalLayerName exists, it's singleton-instance is returned.
	Else, an anonymous instance of CSLayer is returned. These objects are created lazily, tracked in a dictionary and freed when become obsolet.e."
	
	^ (self subclassedLayerNamed: aSymbol)
		ifNotNilDo: #instance
		ifNil: [
			self
				getGlobalInstanceOf: CSLayer
				withName: aSymbol ]! !

!CSLayer class methodsFor: 'private' stamp: 'ag 12/22/2011 15:54'!
new
	
	^ self error: 'Dont create instances of CSLayer (or subclasses) manually.'! !

!CSLayer class methodsFor: 'private' stamp: 'ag 12/23/2011 14:23'!
pvtCreateNamed: aSymbol
	"Only internal, for controlled instance creation."
	
	^ self basicNew initialize: aSymbol; yourself! !

!CSLayer class methodsFor: 'private' stamp: 'ag 12/23/2011 14:32'!
removeGlobalInstanceNamed: aSymbol
	
	^ ((Smalltalk at: aSymbol ifAbsent: [ ^ nil ]) isKindOf: CSLayer)
		ifTrue: [ Smalltalk removeKey: aSymbol ]
		ifFalse: [ self error: 'Global ', aSymbol, ' is not an instance of CSLayer' ]! !

!CSLayer class methodsFor: 'subclassed layers' stamp: 'ag 12/23/2011 14:26'!
resetInstance
	"Reset the layer-instance of this class. Only applicable for subclasses."
	
	self checkIsSubclass.
	^ self removeGlobalInstanceNamed: self globalLayerName! !

!CSLayer class methodsFor: 'layers' stamp: 'ag 12/23/2011 14:26'!
resetLayer: aSymbol
	"Resets the global layer-instance named by aSymbol. Takes layers with their own subclass and anonymous layers into account."
	
	^ (self subclassedLayerNamed: aSymbol)
		ifNotNilDo: #resetInstance
		ifNil: [ self removeGlobalInstanceNamed: aSymbol ]! !

!CSLayer class methodsFor: 'private' stamp: 'ag 12/23/2011 14:11'!
subclassedLayerNamed: aSymbol
	
	^ self allSubclasses
		detect: [ :cl | cl globalLayerName = aSymbol ]
		ifNone: [ nil ]! !

!CSLayer methodsFor: 'queries' stamp: 'ag 12/23/2011 15:14'!
allContainingLayeredMethods
	
	^ CSLayeredMethod allInstances
		select: [ :method |
			method partialMethods
				anySatisfy: [ :partial | partial layers includes: self layerName ] ]! !

!CSLayer methodsFor: 'queries' stamp: 'ag 12/23/2011 15:16'!
allContainingMethods
	"Returns a collection of associations from class to selector."
	
	^ self allContainingLayeredMethods
		collect: [ :layeredMethod | layeredMethod methodClass -> layeredMethod selector ]! !

!CSLayer methodsFor: 'queries' stamp: 'ag 12/23/2011 15:52'!
allContainingPartialMethods
	
	| result |
	result := OrderedCollection new.
	self allContainingLayeredMethods
		do: [ :layeredMethod |
			result addAll:
				(layeredMethod partialMethods
					select: [ :partialMethod | partialMethod layers includes: self layerName ]) ].
	^ result! !

!CSLayer methodsFor: 'initialization' stamp: 'ag 12/22/2011 17:18'!
initialize
	
	self error: 'Please use the #initialize: message with the layer-name as parameter.'.! !

!CSLayer methodsFor: 'initialization' stamp: 'ag 12/22/2011 17:18'!
initialize: aSymbol
	
	super initialize.
	layerName := aSymbol.! !

!CSLayer methodsFor: 'accessing' stamp: 'ag 12/22/2011 17:16'!
layerName
	
	^ layerName! !

!CSLayer methodsFor: 'layer handling' stamp: 'ag 12/23/2011 15:33'!
reset
	"Reset the layer-instance. Next time the layer will be accessed, it will result in another object."
	
	^ self class resetLayer: self layerName! !

CSLayer subclass: #CSTestLayer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2-Tests-Sandbox'!

!CSTestLayer class methodsFor: 'layer instances' stamp: 'ag 12/22/2011 16:40'!
globalLayerName
	
	^ #cstest! !

Object subclass: #CSMethodObject
	instanceVariableNames: 'methodClass selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2'!

CSMethodObject subclass: #CSLayeredMethod
	instanceVariableNames: 'baseMethod partialMethods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2'!

!CSLayeredMethod class methodsFor: 'cleanup' stamp: 'ag 12/23/2011 17:29'!
checkSystemConsistency
	"self checkSystemConsistency"
	
	[ CSMethodObject allInstances
		noneSatisfy: [ :method |
			method selector isNil or: [ method selector isEmpty ]
			or: [ method methodClass isNil ] ]
	] assert.
	
	[ CSProceedMethod allInstances
		noneSatisfy: [ :method |
			method methodClass methodDict includesKey: method selector ]
	] assert.! !

!CSLayeredMethod class methodsFor: 'constants' stamp: 'ag 12/20/2011 21:39'!
pragmaKeyword
	
	^ #layer:! !

!CSLayeredMethod class methodsFor: 'constants' stamp: 'ag 12/23/2011 16:08'!
proceedSelectorBase
	
	^ #proceed! !

!CSLayeredMethod methodsFor: 'accessing' stamp: 'ag 12/17/2011 21:50'!
activeLayers
	
	^ Processor activeProcess csInfo activeLayers! !

!CSLayeredMethod methodsFor: 'accessing' stamp: 'ag 12/23/2011 16:32'!
addPartialMethod: aPartialMethod
	
	aPartialMethod ifNil: [ ^ self ].
	aPartialMethod isBase
		ifTrue: [ baseMethod := aPartialMethod method ] "Ensure normal CompiledMethod"
		ifFalse: [
			self partialMethods
				remove: aPartialMethod ifAbsent: [ "nothing" ];
				add: aPartialMethod ].! !

!CSLayeredMethod methodsFor: 'accessing' stamp: 'ag 1/12/2012 13:00'!
allInvolvedLayers
	
	| layers |
	layers := Set new.
	self partialMethods
		do: [ :each | layers addAll: each layers ].
	^ layers! !

!CSLayeredMethod methodsFor: 'browser' stamp: 'ag 12/21/2011 15:20'!
allPartialMethods
	"Returned collection may not be modified. If receiver includes base-method, it is appended"
	
	^ self includesBaseMethod
		ifTrue: [ self partialMethods copyWith: (CSPartialMethod on: #('<base>') forMethod: self baseMethod) ]
		ifFalse: [ self partialMethods ]! !

!CSLayeredMethod methodsFor: 'accessing' stamp: 'ag 12/20/2011 19:44'!
baseMethod
	
	^ baseMethod! !

!CSLayeredMethod methodsFor: 'browser' stamp: 'ag 12/22/2011 15:20'!
becomeCompiledMethod
	"Remove all layers, keep only the base-method (or one of the partial methods) as normal CompiledMethod.
	This method is not used in the tools, but can be used to remove a certain layered method, or all of them."
	
	self methodClass methodDict
		at: self selector
		put: (self includesBaseMethod ifTrue: [ self baseMethod ] ifFalse: [ self partialMethods anyOne method ]).! !

!CSLayeredMethod methodsFor: 'browser' stamp: 'ag 12/21/2011 11:30'!
getSourceFor: selector in: class
	"Overwritten for ContextBrowser; return the sourcecode for the partial method, that is currently selected."
	
	^ (CSContextBrowser selectedPartialMethods	
		at: self
		ifAbsent: [ self method ])
			getSourceFor: selector in: class! !

!CSLayeredMethod methodsFor: 'accessing' stamp: 'ag 12/20/2011 19:45'!
includesBaseMethod
	
	^ self baseMethod notNil! !

!CSLayeredMethod methodsFor: 'accessing' stamp: 'ag 12/23/2011 16:12'!
isLayered
	"Returns true, if this layered method currently does anything different than a simple CompiledMethod.
	If false, this layered method should be replaced by an equivalent CompiledMethod.
	This is the case, if there are no partial methods."
	
	^ self partialMethods notEmpty! !

!CSLayeredMethod methodsFor: 'accessing' stamp: 'ag 12/20/2011 23:15'!
method
	"Use base method, if possible, else use any partial method. Return nil, if not a single partial method is present."
	
	^ self includesBaseMethod
		ifTrue: [ self baseMethod method ]
		ifFalse: [ self partialMethods ifNotEmpty: [ self partialMethods anyOne ] ]! !

!CSLayeredMethod methodsFor: 'executing' stamp: 'ag 12/23/2011 11:31'!
methodChainFor: activeLayers
	"Chain together all partial methods that are currently active. Chain is ordered in the way the layers have been activated."
	
	| methodChain |
	methodChain := OrderedCollection new.
	activeLayers do: [ :layer |
		methodChain addAll: 
			(self partialMethods
				select: [ :method | (method layers includes: layer) and: [ (methodChain includes: method) not ] ]) ].
	self includesBaseMethod ifTrue: [ methodChain add: self baseMethod ].
	^ methodChain! !

!CSLayeredMethod methodsFor: 'accessing' stamp: 'ag 12/21/2011 10:16'!
partialMethods
	
	^ partialMethods ifNil: [ partialMethods := SortedCollection new ]! !

!CSLayeredMethod methodsFor: 'printing' stamp: 'ag 12/23/2011 15:25'!
printOn: aStream
	
	super printOn: aStream.
	aStream
		nextPutAll: ', ';
		nextPutAll: (self partialMethods size + (self includesBaseMethod ifTrue: [ 1 ] ifFalse: [ 0 ])) asString;
		nextPutAll: ' version(s))'.! !

!CSLayeredMethod methodsFor: 'executing' stamp: 'ag 12/25/2011 18:07'!
proceedSelector
	
	| selector |
	selector := self class proceedSelectorBase.
	self selector isUnary ifTrue: [ ^ selector asSymbol ].
	selector := selector, #':'.
	self selector isInfix ifTrue: [ ^ selector asSymbol ].
	(self selector occurrencesOf: $:) - 1
		timesRepeat: [ selector := selector, #with: ].
	^ selector asSymbol! !

!CSLayeredMethod methodsFor: 'executing' stamp: 'nw 1/22/2012 15:26'!
registerProceed: aReceiver
	"If no ProceedMethod-instance was in the method-dictionary, create one and insert it. If #proceed: was implemented otherwise, throw an error."
	
	(aReceiver class methodDict
		includesKey: self proceedSelector)
		ifTrue: [ | partialMethod |
			partialMethod := aReceiver class methodDict at: self proceedSelector.
			partialMethod class == CSProceedMethod
				ifFalse: [ self error: 'ContextS may not be used with classes implementing #', self proceedSelector ].
			partialMethod incrementRunningLayeredMethods ]
		ifFalse: [
			self flag: #ag. "Access to methodDict should be synchronized on a semaphore. See also #unregisterProceed:"
			aReceiver class methodDict
				at: self proceedSelector
				put: (CSProceedMethod selector: self proceedSelector class: self class) ]. "Default runningLayeredMethods is 1."! !

!CSLayeredMethod methodsFor: 'browser' stamp: 'ag 12/21/2011 15:21'!
removePartialMethod: aPartialMethod
	
	aPartialMethod method == self baseMethod
		ifTrue: [ baseMethod := nil ]
		ifFalse: [ self partialMethods remove: aPartialMethod ].! !

!CSLayeredMethod methodsFor: 'executing' stamp: 'ag 12/23/2011 16:10'!
unregisterProceed: aReceiver
	"We are finished using the ProceedMethod instance. Decrease its counter and remove it from the method-dictionary if necessary."
	
	(aReceiver class methodDict
		at: self proceedSelector
		ifAbsent: [ ^ self ])
		ifNotNilDo: [ :proceedMethod |
			(proceedMethod isKindOf: CSProceedMethod)
				ifTrue: [
					proceedMethod decrementRunningLayeredMethods.
					proceedMethod runningLayeredMethods <= 0
						ifTrue: [ 
							self flag: #ag. "Access to methodDict should be synchronized on a semaphore. See also #registerProceed:"
							aReceiver class methodDict removeKey: self proceedSelector ] ] ].! !

!CSLayeredMethod methodsFor: 'executing' stamp: 'ag 12/23/2011 15:49'!
valueWithReceiver: aReceiver arguments: anArray
	"Memorize old method-chain, compute a new one, kick off the execution and restore the old state afterwards."
	
	| oldProcessState methodChain |
	methodChain := self methodChainFor: self activeLayers.
	methodChain ifEmpty: [ ^ aReceiver ]. "Not a single method to execute... Simulate behavior of an empty method."
	self registerProceed: aReceiver.
	oldProcessState := Processor activeProcess csInfo saveState.
	Processor activeProcess csInfo methodChain: methodChain.
	^ [ methodChain first valueWithReceiver: aReceiver arguments: anArray ]
		ensure: [
			self unregisterProceed: aReceiver.
			Processor activeProcess csInfo restoreStateFrom: oldProcessState ]! !

!CSMethodObject class methodsFor: 'instance creation' stamp: 'ag 12/20/2011 20:12'!
selector: aSymbol class: aClass
	
	^ self new
		selector: aSymbol;
		methodClass: aClass;
		yourself! !

!CSMethodObject methodsFor: 'accessing' stamp: 'ag 1/19/2012 19:01:38.067'!
at: aNumber
	
	^ self method at: aNumber! !

!CSMethodObject methodsFor: 'executing' stamp: 'ag 12/20/2011 21:19'!
doesNotUnderstand: aMessage
	"Required for all meta messages that need information from the original method."
	
	^ self method
		ifNil: [ self ] "Cannot pass the message on, simulate behavior of an empty method."
		ifNotNilDo: [ :msg | msg perform: aMessage selector withArguments: aMessage arguments ]! !

!CSMethodObject methodsFor: 'executing' stamp: 'ag 12/20/2011 19:58'!
method
	"Subclasses have to provide real CompiledMethod-objects as substitute for arbitrary messages. If nil is returned, these messages are dropped."
	
	^ self subclassResponsibility! !

!CSMethodObject methodsFor: 'accessing' stamp: 'ag 12/20/2011 20:11'!
methodClass
	"Answer the value of methodClass"

	^ methodClass! !

!CSMethodObject methodsFor: 'accessing' stamp: 'ag 12/20/2011 20:11'!
methodClass: anObject
	"Set the value of methodClass"

	methodClass := anObject! !

!CSMethodObject methodsFor: 'printing' stamp: 'ag 12/23/2011 15:26'!
printOn: aStream
	
	aStream
		nextPutAll: self class name;
		nextPutAll: '(';
		nextPutAll: (self methodClass name ifNil: [ '' ]);
		nextPutAll: '>>';
		nextPutAll: (self selector ifNil: [ #'' ]).! !

!CSMethodObject methodsFor: 'executing' stamp: 'ag 12/16/2011 11:45'!
run: aSelector with: anArrayOfObjects in: aReceiver
	
	^ self valueWithReceiver: aReceiver arguments: anArrayOfObjects! !

!CSMethodObject methodsFor: 'accessing' stamp: 'ag 12/20/2011 20:11'!
selector
	"Answer the value of selector"

	^ selector! !

!CSMethodObject methodsFor: 'accessing' stamp: 'ag 12/20/2011 20:11'!
selector: anObject
	"Set the value of selector"

	selector := anObject! !

!CSMethodObject methodsFor: 'accessing' stamp: 'ag 1/19/2012 19:01:43.145'!
size
	
	^ self method size! !

!CSMethodObject methodsFor: 'executing' stamp: 'ag 12/20/2011 19:59'!
valueWithReceiver: aReceiver arguments: anArrayOfObjects
	"This message is sent, when the method-object has to execute itself."
	
	^ self subclassResponsibility! !

CSMethodObject subclass: #CSPartialMethod
	instanceVariableNames: 'method layers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2'!

!CSPartialMethod class methodsFor: 'instance creation' stamp: 'ag 12/23/2011 16:49'!
on: aLayer forMethod: aMethod
	
	^ (self
		selector: aMethod selector
		class: aMethod methodClass)
		method: aMethod;
		layers: aLayer;
		yourself! !

!CSPartialMethod methodsFor: 'comparing' stamp: 'ag 12/23/2011 11:31'!
<= otherPartialMethod
	"PartialMethods with fewer layers are sorted first.
	If number of layers is equal, first non-equal layer determines equality. If all are equal, return true."
	
	^ self layers size == otherPartialMethod layers size
		ifTrue: [
			self layers with: otherPartialMethod layers
				do: [ :my :other |
					my = other ifFalse: [ ^ my <= other ] ].
			true ]
		ifFalse: [ self layers size <= otherPartialMethod layers size ]! !

!CSPartialMethod methodsFor: 'comparing' stamp: 'ag 12/23/2011 16:18'!
= anotherPartialMethod
	
	^ anotherPartialMethod class == self class
		and: [ self layers = anotherPartialMethod layers & (self selector = anotherPartialMethod selector) & (self methodClass = anotherPartialMethod methodClass) ]! !

!CSPartialMethod methodsFor: 'printing' stamp: 'ag 12/23/2011 15:28'!
asMenuString
	
	^ String streamContents: [ :stream | self printLayersOn: stream ]! !

!CSPartialMethod methodsFor: 'comparing' stamp: 'ag 12/23/2011 11:31'!
hash
	
	^ self layers hash + self selector hash + self class hash! !

!CSPartialMethod methodsFor: 'accessing' stamp: 'ag 12/23/2011 11:31'!
isBase
	
	^ self layers isNil! !

!CSPartialMethod methodsFor: 'accessing' stamp: 'ag 12/23/2011 11:32'!
layers
	"Answer the value of layer"

	^ layers! !

!CSPartialMethod methodsFor: 'accessing' stamp: 'ag 12/23/2011 11:32'!
layers: anObject
	"Set the value of layer"

	layers := anObject! !

!CSPartialMethod methodsFor: 'accessing' stamp: 'mp 6/13/2010 14:19'!
method
	"Answer the value of method"

	^ method! !

!CSPartialMethod methodsFor: 'accessing' stamp: 'mp 6/13/2010 14:19'!
method: anObject
	"Set the value of method"

	method := anObject! !

!CSPartialMethod methodsFor: 'printing' stamp: 'ag 12/23/2011 15:28'!
printLayersOn: stream
	
	self layers
		do: [ :each | stream nextPutAll: each ]
		separatedBy: [ stream nextPut: Character space ].! !

!CSPartialMethod methodsFor: 'printing' stamp: 'ag 12/23/2011 15:28'!
printOn: aStream
	
	super printOn: aStream.
	aStream nextPutAll: ', layer(s) '.
	self printLayersOn: aStream.
	aStream nextPutAll: ')'.! !

!CSPartialMethod methodsFor: 'executing' stamp: 'ag 12/21/2011 00:09'!
valueWithReceiver: aReceiver arguments: anArray
	"Memorize in the process-state, that the current method has been started; execute the backing CompiledMethod in the receivers context."
	
	Processor activeProcess csInfo executedPartialMethod: self.
	^ [ aReceiver withArgs: anArray executeMethod: self method ]
		ensure: [ Processor activeProcess csInfo executedPartialMethod: nil ] "Allow garbage collection of this partial method"! !

CSMethodObject subclass: #CSProceedMethod
	instanceVariableNames: 'runningLayeredMethods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2'!

!CSProceedMethod class methodsFor: 'clean up' stamp: 'ag 12/23/2011 17:15'!
cleanUp
	"Remove all leftover instances of ProceedMethod in method-dictionaries of classes.
	ProceedMethod-instances are removed inside ensure:-blocks, but nevertheless sometimes some of them can stay somewhere in method-dictionaries..."
	
	| number |
	number := 0.
	CSProceedMethod allInstancesDo: [ :proceed |
			(proceed methodClass methodDict includesKey: proceed selector)
				ifTrue: [
					number := number + 1.
					proceed methodClass methodDict removeKey: proceed selector ] ].
	^ number! !

!CSProceedMethod methodsFor: 'accessing' stamp: 'ag 12/20/2011 21:01'!
decrementRunningLayeredMethods
	
	runningLayeredMethods := self runningLayeredMethods - 1.! !

!CSProceedMethod methodsFor: 'accessing' stamp: 'ag 12/20/2011 21:09'!
incrementRunningLayeredMethods
	
	runningLayeredMethods := self runningLayeredMethods + 1.! !

!CSProceedMethod methodsFor: 'executing' stamp: 'ag 12/20/2011 20:58'!
method
	"There is no real CompiledMethod to pass incoming messages to."
	
	^ nil! !

!CSProceedMethod methodsFor: 'executing' stamp: 'ag 12/23/2011 15:55'!
nextPartialMethod
	"Based on executedPartialMethod, find the next method in the methodChain."
	
	| info index |
	info := Processor activeProcess csInfo.
	index := info methodChain indexOf: info executedPartialMethod.
	info methodChain size <= index ifTrue: [ ^ nil ]. "Method chain has been executed to the end."
	^ info methodChain at: index + 1! !

!CSProceedMethod methodsFor: 'printing' stamp: 'ag 12/23/2011 15:31'!
printOn: aStream
	
	super printOn: aStream.
	aStream
		nextPutAll: ', count ';
		nextPutAll: (runningLayeredMethods ifNil: [ 0 ]) asString;
		nextPut: $).! !

!CSProceedMethod methodsFor: 'accessing' stamp: 'ag 12/20/2011 21:11'!
runningLayeredMethods
	"The default value is 1 to account for the layered method that created this proceed-method."
	
	^ runningLayeredMethods ifNil: [ runningLayeredMethods := 1 ]! !

!CSProceedMethod methodsFor: 'executing' stamp: 'nw 1/19/2012 19:22'!
valueWithReceiver: aReceiver arguments: anArray
	
	^ self nextPartialMethod
		ifNil: [ aReceiver ] "No method to proceed to. Simulate behavior of an empty method. Either the base method has called proceed: or there is no base method."
		ifNotNilDo: [ :partialMethod |
			aReceiver withArgs: anArray executeMethod: partialMethod ]! !

Object subclass: #CSMyClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2-Tests-Sandbox'!

!CSMyClass methodsFor: 'as yet unclassified' stamp: ''!
bla! !

!CSMyClass methodsFor: 'as yet unclassified' stamp: ''!
blub
	"BASE"! !

!CSMyClass methodsFor: 'as yet unclassified' stamp: ''!
no
	self blub.! !

!CSMyClass methodsFor: 'as yet unclassified' stamp: ''!
partialOnly

	<layer: #l1>! !

CSMyClass subclass: #CSMyClassSub
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2-Tests-Sandbox'!

!CSMyClassSub methodsFor: 'as yet unclassified' stamp: ''!
blub

	"neu in sub"! !

!CSMyClassSub methodsFor: 'as yet unclassified' stamp: ''!
no
! !

!CSMyClassSub methodsFor: 'as yet unclassified' stamp: ''!
partialOnly
	<layer: #l4>
	self huhu.
	super partialOnly.! !

Object subclass: #CSProcessContextInformation
	instanceVariableNames: 'activeLayers methodChain executedPartialMethod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ContextS2'!

!CSProcessContextInformation methodsFor: 'accessing' stamp: 'ag 12/20/2011 20:00'!
activeLayers
	^ activeLayers ifNil: [ activeLayers := #() ]! !

!CSProcessContextInformation methodsFor: 'accessing' stamp: 'nw 12/12/2011 17:05'!
activeLayers: layers
	activeLayers := layers! !

!CSProcessContextInformation methodsFor: 'accessing' stamp: 'nw 12/12/2011 17:06'!
executedPartialMethod
	^ executedPartialMethod! !

!CSProcessContextInformation methodsFor: 'accessing' stamp: 'nw 12/12/2011 17:06'!
executedPartialMethod: partialMethod
	executedPartialMethod := partialMethod! !

!CSProcessContextInformation methodsFor: 'accessing' stamp: 'ag 12/18/2011 11:13'!
methodChain
	
	^ methodChain! !

!CSProcessContextInformation methodsFor: 'accessing' stamp: 'ag 12/18/2011 11:13'!
methodChain: anOrderedCollection
	
	methodChain := anOrderedCollection.! !

!CSProcessContextInformation methodsFor: 'layer handling' stamp: 'ag 12/23/2011 15:48'!
restoreStateFrom: anArray
	"anArray must be the return value of an earlier #saveState call."
	
	self methodChain: anArray first.
	self executedPartialMethod: anArray second.! !

!CSProcessContextInformation methodsFor: 'layer handling' stamp: 'ag 12/23/2011 15:48'!
saveState
	
	^ { self methodChain. self executedPartialMethod }! !

!CSProcessContextInformation methodsFor: 'layer handling' stamp: 'ag 12/20/2011 23:54'!
withLayers: aCollection do: aBlock
	"Convenience method for layer-manipulation. Stores old layers and restores them after executing aBlock."
	
	| oldLayers |
	oldLayers := self activeLayers.
	self activeLayers: aCollection.
	^ aBlock ensure: [ self activeLayers: oldLayers ]! !

!Object methodsFor: '*ContextS2' stamp: 'ag 12/23/2011 11:30'!
currentLayer
	"Return the first of the currentLayers for convenience.
	This should be the first activated layer that has lead to the activation of the last partial method."
	
	^ self currentLayers ifNotNilDo: #first! !

!Object methodsFor: '*ContextS2' stamp: 'ag 12/23/2011 11:33'!
currentLayers
	"Return the layers of the current execution. If no method associated to any layer is on the execution stack, nil is returned.
	Else, the last partial method that has started execution determines the 'current layers'.
	These are the layers that have lead to the activation of this partial method (-> the subset of the currently active layers, that intersects with the partial methods layers).
	This returns (sub)instances of CSLayer.
	If a subclass of CSLayer with the right globalLayerName exists, its singleton-instance is used, else an anonymous instance of CSLayer."
	
	| info |
	info := Processor activeProcess csInfo.
	^ info executedPartialMethod
		ifNotNilDo: [ :partialMethod |
			((info activeLayers intersection: partialMethod layers)
				collect: [ :layername | CSLayer layerNamed: layername ])
					ifEmpty: [ nil ] ]! !

!Array methodsFor: '*ContextS2' stamp: 'ag 12/20/2011 21:29'!
withLayersDo: aBlock
	"Memorize currently active layers, add the new ones and restore the state afterwards."
	
	^ Processor activeProcess csInfo
		withLayers: (Processor activeProcess csInfo activeLayers copyWithoutAll: self), self
		do: aBlock! !

!Array methodsFor: '*ContextS2' stamp: 'ag 12/20/2011 21:31'!
withoutLayersDo: aBlock
	"Memorize currently active layers, remove some and restore the state afterwards."
	
	^ Processor activeProcess csInfo
		withLayers: (Processor activeProcess csInfo activeLayers copyWithoutAll: self)
		do: aBlock! !
